@@ -3,6 +3,7 @@
 
 # Copyright (C) 2008, 2009 Russ Allbery
 # Copyright (C) 2008 Frank Lichtenheld
+# Copyright (C) 2012 Kees Cook
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
@@ -27,7 +28,7 @@ use Lintian::Relation;
 use Carp qw(croak);
 use Parse::DebianChangelog;
 
-use Util;
+use Lintian::Util qw(fail open_gz);
 
 # Initialize a new binary package collect object.  Takes the package name,
 # which is currently unused.
@@ -89,17 +90,19 @@ sub control_index {
     return $self->_fetch_index_data('control-index', 'control-index');
 }
 
-# Returns sorted file info (eqv to sort keys %{$info->file_info'}), except it is cached.
-#  sub sorted_file_info Needs-Info file-info
-sub sorted_file_info{
-    my ($self) = @_;
-    my $info;
-    my @result;
-    return $self->{sorted_file_info} if exists $self->{sorted_file_info};
-    $info = $self->file_info();
-    @result = sort keys %{$info};
-    $self->{sorted_file_info} = \@result;
-    return \@result;
+# Returns a handle with the strings in a given binary file (as computed
+# by coll/strings)
+#
+# sub strings Needs-Info strings
+sub strings {
+    my ($self, $file) = @_;
+    my $real = $self->_fetch_extracted_dir ('strings', 'strings', $file);
+    if ( not -f "${real}.gz" ) {
+        open my $fd, '<', '/dev/null';
+        return $fd;
+    }
+    my $fd = open_gz ("$real.gz") or fail "open ${file}.gz: $!";
+    return $fd;
 }
 
 # Returns the md5sums as calculated by the md5sums collection
@@ -133,6 +136,7 @@ sub scripts {
     return $self->{scripts} if exists $self->{scripts};
     my $base_dir = $self->base_dir();
     my %scripts;
+    local $_;
     # sub scripts Needs-Info scripts
     open(SCRIPTS, '<', "$base_dir/scripts")
         or fail("cannot open scripts $base_dir/file: $!");
@@ -163,15 +167,16 @@ sub objdump_info {
     my $base_dir = $self->base_dir();
     my %objdump_info;
     my ($dynsyms, $file);
+    local $_;
     # sub objdump_info Needs-Info objdump-info
-    open(my $idx, '<', "$base_dir/objdump-info")
-        or fail("cannot open $base_dir/objdump-info: $!");
-    while (<$idx>) {
+    open my $fd, '-|', 'gzip', '-dc', "$base_dir/objdump-info.gz"
+        or fail "cannot open $base_dir/objdump-info.gz: $!";
+    while (<$fd>) {
         chomp;
 
         next if m/^\s*$/o;
 
-        if (m,^-- \./(.+)$,) {
+        if (m,^-- (?:\./)?(.+)$,) {
             if ($file) {
                 $objdump_info{$file->{name}} = $file;
             }
@@ -205,9 +210,9 @@ sub objdump_info {
                 push @{$file->{NOTES}}, 'File format not recognized';
             } elsif (m/^objdump: .*?: File truncated$/) {
                 push @{$file->{NOTES}}, 'File truncated';
-            } elsif (m/^objdump: \..*?: Packed with UPX$/) {
+            } elsif (m/^objdump: .*?: Packed with UPX$/) {
                 push @{$file->{NOTES}}, 'Packed with UPX';
-            } elsif (m/objdump: \..*?: Invalid operation$/) {
+            } elsif (m/objdump: .*?: Invalid operation$/) {
                 # Don't anchor this regex since it can be interspersed with other
                 # output and hence not on the beginning of a line.
                 push @{$file->{NOTES}}, 'Invalid operation';
@@ -225,8 +230,6 @@ sub objdump_info {
                 if (defined $file->{STACK} and $file->{STACK} eq '0') {
                     m/\sflags\s+(\S+)/o;
                     $file->{STACK} = $1;
-                } else {
-                    $file->{OTHER_DATA} = 1;
                 }
             }
         }
@@ -236,44 +239,82 @@ sub objdump_info {
     }
     $self->{objdump_info} = \%objdump_info;
 
+    close $fd;
+
     return $self->{objdump_info};
 }
 
 
+# Returns the information from collect/hardening-info
+# sub hardening_info Needs-Info hardening-info
+sub hardening_info {
+    my ($self) = @_;
+    return $self->{hardening_info} if exists $self->{hardening_info};
+    my $base_dir = $self->base_dir();
+    my %hardening_info;
+    my ($file);
+    local $_;
+    open(my $idx, '<', "$base_dir/hardening-info")
+        or fail("cannot open $base_dir/hardening-info: $!");
+    while (<$idx>) {
+        chomp;
+
+        if (m,^([^:]+):(?:\./)?(.*)$,) {
+            my ($tag, $file) = ($1, $2);
+            push(@{$hardening_info{$file}}, $tag);
+        }
+    }
+
+    $self->{hardening_info} = \%hardening_info;
+
+    return $self->{hardening_info};
+}
+
+
 # Returns the information from collect/objdump-info
 # sub java_info Needs-Info java-info
 sub java_info {
     my ($self) = @_;
     return $self->{java_info} if exists $self->{java_info};
 
+    my $base_dir = $self->base_dir;
     my %java_info;
-    open(my $idx, '<', 'java-info')
-        or fail("cannot open java-info: $!");
+    if ( ! -f "$base_dir/java-info.gz" ) {
+        # no java-info.gz => no jar files to collect data.  Just
+        # return an empty hash ref.
+        $self->{java_info} = \%java_info;
+        return $self->{java_info};
+    }
+
+    open my $idx, '-|', 'gzip', '-dc', "$base_dir/java-info.gz"
+        or fail "cannot open $base_dir/java-info.gz: $!";
     my $file;
-    my $file_list = 0;
+    my $file_list;
     my $manifest = 0;
+    local $_;
     while (<$idx>) {
         chomp;
         next if m/^\s*$/o;
 
-        if (m#^-- \./(.+)$#o) {
-            $file = $1;
-            $java_info{$file}->{files} = [];
-            $file_list = $java_info{$file}->{files};
-            $manifest = 0;
-        }
-        elsif (m#^-- MANIFEST: \./(?:.+)$#o) {
+        if (m#^-- MANIFEST: (?:\./)?(?:.+)$#o) {
             # TODO: check $file == $1 ?
             $java_info{$file}->{manifest} = {};
             $manifest = $java_info{$file}->{manifest};
             $file_list = 0;
         }
+        elsif (m#^-- (?:\./)?(.+)$#o) {
+            $file = $1;
+            $java_info{$file}->{files} = {};
+            $file_list = $java_info{$file}->{files};
+            $manifest = 0;
+        }
         else {
             if($manifest && m#^  (\S+):\s(.*)$#o) {
                 $manifest->{$1} = $2;
             }
-            elsif($file_list) {
-                push @{$file_list}, $_;
+            elsif ($file_list) {
+                my ($fname, $clmajor) = (m#^(.*):\s*([-\d]+)$#);
+                $file_list->{$fname} = $clmajor;
             }
 
         }
@@ -298,22 +339,18 @@ sub relation {
                    weak   => [ qw(recommends suggests) ]);
     my $result;
     if ($special{$field}) {
-        my $merged;
-        for my $f (@{ $special{$field} }) {
-            my $value = $self->field($f);
-            $merged .= ', ' if (defined($merged) and defined($value));
-            $merged .= $value if defined($value);
-        }
-        $result = $merged;
+        $result = Lintian::Relation->and (
+            map { $self->relation ($_) } @{ $special{$field} }
+        );
     } else {
         my %known = map { $_ => 1 }
             qw(pre-depends depends recommends suggests enhances breaks
                conflicts provides replaces);
         croak("unknown relation field $field") unless $known{$field};
         my $value = $self->field($field);
-        $result = $value if defined($value);
+        $result = Lintian::Relation->new ($value);
     }
-    $self->{relation}->{$field} = Lintian::Relation->new($result);
+    $self->{relation}->{$field} = $result;
     return $self->{relation}->{$field};
 }
 
@@ -494,6 +531,11 @@ package.
 
 This is based on the package's description.
 
+=item strings (FILE)
+
+Returns an open handle, which will read the data from coll/strings for
+FILE.
+
 =back
 
 =head1 AUTHOR
