@@ -53,9 +53,6 @@ my %SUPPORTED_TYPES = (
     'udeb'    => 1,
 );
 
-
-
-# Export now due to cicular depends between Lab and Lab::Package.
 our (@EXPORT, @EXPORT_OK, %EXPORT_TAGS);
 
 BEGIN {
@@ -68,11 +65,12 @@ BEGIN {
     );
 };
 
-use Util qw(delete_dir get_dsc_info);
-
 use Lintian::Collect;
 use Lintian::Lab::Entry;
 use Lintian::Lab::Manifest;
+use Lintian::Util qw(delete_dir get_dsc_info);
+
+=encoding utf8
 
 =head1 NAME
 
@@ -91,7 +89,7 @@ Lintian::Lab -- Interface to the Lintian Lab
  $lab->open;
  
  # Fetch a package from the lab
- my $pkg = $lab->get_package ('lintian', 'binary', '2.5.4', 'all');
+ my $lpkg = $lab->get_package ('lintian', 'binary', '2.5.4', 'all');
  
  #FIXME: Add more to the synopsis here
  
@@ -99,6 +97,10 @@ Lintian::Lab -- Interface to the Lintian Lab
 
 =head1 DESCRIPTION
 
+This module provides an abstraction from "How and where" packages are
+placed.  It handles creation and deletion of the Lintian Lab itself as
+well as providing access to the entries.
+
 =head2 Methods
 
 =over 4
@@ -153,8 +155,8 @@ sub new {
 Returns the absolute path to the base of the lab.
 
 Note: This may return the empty string if either the lab has been
-deleted or this is a temporary lab that has not been created yet.
-In the latter case, $lab->create should be run to get a
+deleted or this is a temporary lab that has not been created yet.  In
+the latter case, $lab->create or $lab->open should be run to get a
 non-empty value from this method.
 
 =item $lab->is_open
@@ -194,19 +196,19 @@ sub exists {
 
 Fetches an existing package from the lab.
 
-The first argument can be a L<Lintian::Processable|proccessable>.  In that
+The first argument can be a L<processable|Lintian::Processable>.  In that
 case all other arguments are ignored.
 
-If the first calling convention is used then this method will by
-default search for an existing package.  The @extra argument can be
-used to narrow the search or even to add a new entry.
+If the first calling convention is used then this method will search
+for an existing package.  The @extra argument can be used to narrow
+the search or even to add a new entry.
 
 @extra consists of (in order):
  - version
  - arch (Ignored if $pkg_type is "source")
 
-If version or arch is omitted (or undef) then that search parameter is
-consider a wildcard for "any".  Example:
+If version or arch is omitted (or if it is undef) then that search
+parameter is consider a wildcard for "any".  Example:
 
  # Returns all eclipse-platform packages with architecture i386 regardless
  # of their version (if any)
@@ -222,11 +224,18 @@ consider a wildcard for "any".  Example:
 In list context, this returns a list of matches.  In scalar context
 this returns the first match (if any).
 
+If the second calling convention is used, then this method will search
+for an entry matching the the processable passed.  If such an entry
+does not exists, an new "non-existing" L<entry|Lintian::Lab::Entry>
+will be returned.  This entry can be created by using the "create"
+method on the entry.
+
 =cut
 
 sub get_package {
-    my ($self, $pkg, $pkg_type, $pkg_version, $pkg_arch, $pkg_path) = @_;
+    my ($self, $pkg, $pkg_type, $pkg_version, $pkg_arch) = @_;
     my $pkg_name;
+    my $pkg_path;
     my @entries;
     my $index;
     my $proc;
@@ -265,7 +274,7 @@ sub get_package {
             $e = $index->get (@keys);
             return unless $e;
             $pkg_src = $e->{'source'};
-            $pkg_src_version = $e->{'source-version'}//$pkg_version;
+            $pkg_src_version = $e->{'source-version'};
         }
         $dir = $self->_pool_path ($pkg_src, $pkg_type, $pkg_name, $pkg_version, $pkg_arch);
         push @entries, Lintian::Lab::Entry->_new ($self, $pkg_name, $pkg_version, $pkg_arch, $pkg_type, $pkg_path, $pkg_src, $pkg_src_version, $dir);
@@ -285,7 +294,7 @@ sub get_package {
             $pp = $entry->{'file'};
             $pkg_src = $entry->{'source'};
             $dir = $self->_pool_path ($pkg_src, $pkg_type, $pkg_name, $v, $a);
-            push @entries,  Lintian::Lab::Entry->_new ($self, $pkg_name, $v, $a, $pkg_type, $pp, $pkg_src, $entry->{'source-version'}//$v, $dir);
+            push @entries,  Lintian::Lab::Entry->_new ($self, $pkg_name, $v, $a, $pkg_type, $pp, $pkg_src, $entry->{'source-version'}, $dir);
         };
         my @sk = ($pkg_name);
         push @sk, $pkg_version if defined $pkg_version;
@@ -300,6 +309,10 @@ sub get_package {
 Passes each lab entry to $visitor.  If $pkg_type is passed, then only
 entries of that type are passed.
 
+The visitor is given a reference to the L<entry|Lintian::Lab::Entry>,
+the package name, the package version and the package architecture
+(may be undef for source packages).
+
 =cut
 
 sub visit_packages {
@@ -313,10 +326,9 @@ sub visit_packages {
             my ($me, $pkg_name, $pkg_version, $pkg_arch) = @_;
             my $pkg_src = $me->{'source'}//$pkg_name;
             my $dir = $self->_pool_path ($pkg_src, $pkg_type, $pkg_name, $pkg_version, $pkg_arch);
-            my $pp = $me->{'file'};
-            my $pkg_src_version = $me->{'source-version'}//$pkg_version;
+            my $pkg_src_version = $me->{'source-version'};
             my $lentry = Lintian::Lab::Entry->_new ($self, $pkg_name, $pkg_version, $pkg_arch,
-                                                   $pkg_type, $pp, $pkg_src, $pkg_src_version, $dir);
+                                                   $pkg_type, undef, $pkg_src, $pkg_src_version, $dir);
             $visitor->($lentry, $pkg_name, $pkg_version, $pkg_arch);
         };
         $index->visit_all ($intv);
@@ -399,15 +411,14 @@ sub _pool_path {
     return "$dir/pool/$p";
 }
 
-=item lab->generate_diffs(@lists)
+=item $lab->generate_diffs(@lists)
 
 Each member of @lists must be a Lintian::Lab::Manifest.
 
 The lab will generate a diff between the given member and its state
-for the given package type.  The diffs are returned in the same order
-as they appear in @lists.
+for the given package type.
 
-The diffs are valid until the original list is modified or a package
+The diffs are valid until the original manifest is modified or a package
 is added or removed to the lab.
 
 =cut
@@ -427,6 +438,76 @@ sub generate_diffs {
     return @diffs;
 }
 
+=item $lab->repair_lab ()
+
+Checks the lab contents against the current meta-data and syncs them.
+The lab must be open and should not be access while this method is
+running.
+
+This returns the number of corrections done by this process.  If there
+were any corrections, the state files are written before returning.
+
+The method may croak if it is unable to do a full check of the lab or
+if it is unable to write the corrected metadata.
+
+Note: This may (and generally will) correct "broken" entries by
+removing them.
+
+=cut
+
+sub repair_lab {
+    my ($self) = @_;
+    my $updates = 0;
+    croak "Lab is not open.\n" unless $self->is_open;
+    foreach my $pkg_type (keys %SUPPORTED_TYPES) {
+        my $index = $self->_get_lab_index ($pkg_type);
+        my $visitor = sub {
+            my ($metadata, @keys) = @_;
+            my ($pkg_name, $pkg_version, $pkg_arch) = @keys;
+            my $pkg_src = $metadata->{'source'}//$pkg_name;
+            my $pkg_src_version = $metadata->{'source-version'};
+            my $dir = $self->_pool_path ($pkg_src, $pkg_type, $pkg_name, $pkg_version, $pkg_arch);
+            my $entry;
+            unless ( -d $dir && -f "$dir/.lintian-status") {
+                # The entry is clearly not here, remove it from the metadata
+                $index->delete (@keys);
+                $updates++;
+                -d $dir && rmdir $dir;
+                return;
+            }
+            eval {
+                $entry = Lintian::Lab::Entry->_new ($self, $pkg_name, $pkg_version, $pkg_arch,
+                                                    $pkg_type, undef, $pkg_src, $pkg_src_version,
+                                                    $dir);
+            };
+            unless ($entry && $entry->exists) {
+                # We either cannot load the entry or it does not
+                # believe it exists - either way, the metadata is out
+                # of date.
+                if ($entry) {
+                    $entry->remove;
+                    # Strictly speaking $entry->remove ought to clean
+                    # up the index for us, but fall through and do it
+                    # anyway.
+                } else {
+                    # The entry is not here to clean up, lets purge it
+                    # the good old fashioned way.
+                    delete_dir ($dir);
+                }
+                $index->delete (@keys);
+                $updates++;
+            }
+        };
+
+        $index->visit_all ($visitor);
+    }
+
+    # FIXME: scan the pool for entries not in the metadata.
+
+    $self->_write_manifests;
+
+    return $updates;
+}
 
 =item $lab->create ([$opts])
 
@@ -434,6 +515,9 @@ Creates a new lab.  It will create $lab->dir if it does not exist.
 It will also create a basic empty lab.  If this is a temporary
 lab, this method will also setup the temporary dir for the lab.
 
+The $lab will I<not> be opened by this method.  This should be done
+afterwards by invoking the open method.
+
 B<$opts> (if present) is a hashref containing options.  The following
 options are accepted:
 
@@ -472,14 +556,13 @@ sub create {
 
     $opts = {} unless $opts;
     $mode = $opts->{'mode'} if exists $opts->{'mode'};
-    if ( !$dir or $self->{'mode'} eq LAB_MODE_TEMP) {
-        if ($self->{'mode'} eq LAB_MODE_TEMP) {
+    if ( !$dir or $self->is_temp) {
+        if ($self->is_temp) {
             my $keep = $opts->{'keep-lab'}//0;
-            my $topts = { CLEAN => !$keep, TMPDIR => 1 };
-            my $t = tempdir ('temp-lintian-lab-XXXXXX', $topts);
+            my %topts = ( 'CLEANUP' => !$keep, 'TMPDIR' => 1 );
+            my $t = tempdir ('temp-lintian-lab-XXXXXXXXXX', %topts);
             $dir = Cwd::abs_path ($t);
-            # Should the croak() be using $t rather than $dir?
-            croak "Could not resolve $dir: $!" unless $dir;
+            croak "Could not resolve $t: $!" unless $dir;
             $self->{'dir'} = $dir;
             $self->{'keep-lab'} = $keep;
         } else {
@@ -549,11 +632,11 @@ sub create {
 =item $lab->open
 
 Opens the lab and reads the contents into caches.  If the lab is
-temporary this will create a temporary dir to store the contents of
-the lab.
+temporary and does not exists, this method will call create to
+initialize the temporary lab.
 
 This will croak if the lab is already open.  It may also croak for
-the same reasons as $lab->create if this is a temporary lab.
+the same reasons as $lab->create if the lab is temporary (see above).
 
 Note: for static labs, $lab->dir must point to an existing consistent
 lab or this will croak.  To open a new lab, please use
@@ -561,7 +644,7 @@ $lab->create.
 
 Note: It is not possible to pass options to the creation of the
 temporary lab.  If special options are required, please use
-$lab->create.
+$lab->create directly.
 
 =cut
 
@@ -570,7 +653,7 @@ sub open {
     my $dir;
     my $msg = "Open Lab failed";
     croak ('Lab is already open') if $self->is_open;
-    if ($self->{'mode'} eq LAB_MODE_TEMP) {
+    if ($self->is_temp) {
         $self->create unless $self->exists;
         $dir = $self->dir;
     } else {
@@ -589,7 +672,7 @@ sub open {
     }
 
     # Check the lab-format - this ought to be redundant for temp labs, but
-    # it's simpler to do it that way.
+    # it's simpler to do it for them anyway.
     my $header = get_dsc_info ("$dir/info/lab-info");
     my $format = $header->{'lab-format'}//'';
     my $layout = $header->{'layout'}//'pool';
@@ -597,8 +680,9 @@ sub open {
         croak "$msg: Lab format $format is not supported ($dir)" if $format;
         croak "$msg: No lab format specification in $dir/info/lab-info";
     }
-    if ($layout && lc($layout) ne 'pool') {
+    unless ($layout && lc($layout) eq 'pool') {
         # Unknown layout style?
+        croak "$msg: Layout field present but with no value" unless $layout;
         croak "$msg: Implementation does not support the layout \"$layout\"";
     }
 
@@ -622,15 +706,11 @@ was created with "keep-lab" (see $lab->create).
 sub close {
     my ($self) = @_;
     return unless $self->exists;
-    if ($self->{'mode'} eq LAB_MODE_TEMP && !$self->{'keep-lab'}) {
+    if ($self->is_temp && !$self->{'keep-lab'}) {
         # Temporary lab (without "keep-lab" property)
         $self->remove;
     } else {
-        my $dir = $self->dir;
-        while ( my ($pkg_type, $plist) = (each %{ $self->{'state'} }) ) {
-            # write_list croaks on error, so no need for "or croak/die"
-            $plist->write_list ("$dir/info/${pkg_type}-packages");
-        }
+        $self->_write_manifests;
     }
     $self->{'state'} = {};
     $self->{'is_open'} = 0;
@@ -638,6 +718,16 @@ sub close {
     return 1;
 }
 
+sub _write_manifests {
+    my ($self) = @_;
+    my $dir = $self->dir;
+    while ( my ($pkg_type, $plist) = (each %{ $self->{'state'} }) ) {
+        # write_list croaks on error, so no need for "or croak/die"
+        $plist->write_list ("$dir/info/${pkg_type}-packages")
+            if $plist->dirty;
+    }
+}
+
 =item $lab->remove
 
 Removes the lab and everything in it.  Any reference to an entry
@@ -695,7 +785,7 @@ sub remove {
     }
 
     # dynamic lab?
-    if ($self->{'mode'} eq LAB_MODE_TEMP) {
+    if ($self->is_temp) {
         rmdir $dir or croak "rmdir $dir: $!";
         $self->{'dir'} = '';
     }
@@ -704,6 +794,20 @@ sub remove {
     return 1;
 }
 
+=item $lab->is_temp
+
+Returns a truth value if $lab is a temporary lab.
+
+Note: This returns a truth value, even if the lab was created with the
+"keep-lab" property.
+
+=cut
+
+sub is_temp {
+    my ($self) = @_;
+    return $self->{'mode'} eq LAB_MODE_TEMP ? 1 : 0;
+}
+
 # initialize the instance
 #
 # May be overriden by a sub-class.
@@ -780,7 +884,7 @@ sub _entry_created {
             $area = $1;
         }
         $data{'architecture'}   = $entry->pkg_arch;
-        $data{'area'}           = 'main';
+        $data{'area'}           = $area;
         $data{'package'}        = $pkg_name;
         $data{'source'}         = $entry->pkg_src;
         $data{'source-version'} = $entry->pkg_src_version;
@@ -803,13 +907,19 @@ In lab format 11 the lab format is stored in $LAB/info/lab-info.  The
 rest of the files in $LAB/info/* have been re-purposed to be a list of
 packages in the lab.
 
-The $LAB/info/lab-info file also contains modifying parameters.  All
-parameters are matched case-insensitively and the accepted parameters
-are:
+The $LAB/info/lab-info is parsed as a debian control file (See Debian
+Policy Manual §5.1 for syntax).  The consists of a single paragraph
+and only the following fields are allowed:
 
 =over 4
 
-=item Layout
+=item Lab-Format (simple, mandatory)
+
+This field contains the lab format of this lab.  Generally this is
+simply an integer (though during development non-integers have been
+used).
+
+=item Layout (simple, optional)
 
 The layout parameter describes how packages are stored in the lab.
 Currently the only accepted value is "pool" and the value is not
@@ -829,6 +939,9 @@ If the field is missing, it defaults to "pool".
 
 =back
 
+It is allowed to use comments in $LAB/info/lab-info as described
+in the Debian Policy Manual §5.1.
+
 =head1 AUTHOR
 
 Niels Thykier <niels@thykier.net>
